;===============================================================================
;                'JUMPING JACK' (ZX SPECTRUM) - DISASSEMBLY
;                 Chris Ward, 2012 (cjw116@googlemail.com)
;                 Original game (c) 1983 Imagine Software
;===============================================================================
;
; This is file 2 of 4:
;
;    1) JJLoader.txt       Commented RAM disassembly of loader program with all
;                          addresses & object code
;
;   *2) JJLoader.asm       Address-independent assembler source file of above -
;                          can be reassembled at any address
;
;    3) JJGame.txt         Commented RAM disassembly of game with all addresses
;                          & object code. Also a list of useful POKES.
;
;    4) JJGame.asm         Address-independent assembler source file of above -
;                          can be reassembled at any address
;
;
; NB: All mistakes are mine. Please forgive the amount of comments and the way
; the files are formatted. I've never disassembled anything before and this
; worked for me! Hope it is of interest to someone.
;
; The .asm files have been tested with Pasmo (http://pasmo.speccy.org/) and
; they produce exact binary matches of the original.
;
; Enjoy!
;
;
;===============================================================================

; This will assemble to create a 2700 byte binary - identical to the code in
; the BASIC Line 1 REM statement.


ROM_BEEP       EQU     03b5h
ROM_LOAD       EQU     0556h

               ORG     5cd0h       ; NB: Program start is at ORG + 506h (1286d)


; DATA - GFX definitions of the big letters.
;
; Each big letter is made up of a rectangle of cells (each cell 8x8 pixels).
; Each cell is defined by 8 bytes - each byte represeting the bit definitions
; of the 8 pixels in each row.

; "J" (Width 3, Height 5)

JData:         db      00000000b   ; Cell (0,0)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00111111b   ; Cell (0,1)
               db      00111111b
               db      11111100b
               db      11111100b
               db      11111111b
               db      11111111b
               db      11110000b
               db      11110000b

               db      00000000b   ; Cell (0,2)
               db      00000000b
               db      11000000b
               db      11000000b
               db      11110000b
               db      11110000b
               db      11000000b
               db      11000000b

               db      00000000b   ; Cell (1,0)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00111111b
               db      00111111b
               db      00000000b
               db      00000000b

               db      00111111b   ; Cell (1,1)
               db      00111111b
               db      00001100b
               db      00001100b
               db      11111111b
               db      11111111b
               db      00001100b
               db      00001100b

               db      00000000b   ; Cell (1,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      11111111b
               db      11111111b
               db      00000000b
               db      00000000b

               db      00000000b   ; Cell (2,0)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00001100b   ; Cell (2,1)
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b

               db      00000000b   ; Cell (2,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00000000b   ; Cell (3,0)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00001100b   ; Cell (3,1)
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b

               db      00000000b   ; Cell (3,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00000000b   ; Cell (4,0)
               db      00000000b
               db      00000000b
               db      00000000b
               db      11111111b
               db      11111111b
               db      11000000b
               db      11000000b

               db      00001100b   ; Cell (4,1)
               db      00001100b
               db      00001100b
               db      00001100b
               db      11111100b
               db      11111100b
               db      00000000b
               db      00000000b

               db      00000000b   ; Cell (4,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

; "U" (Width 3, Height 3)

UData:         db      00000000b   ; Cell (0,0)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00000000b   ; Cell (0,1)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b

               db      00000000b   ; Cell (0,2)
               db      00000000b
               db      00001111b
               db      00001111b
               db      00000011b
               db      00000011b
               db      00000011b
               db      00000011b

               db      00000000b   ; Cell (1,0)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00001100b
               db      00001100b
               db      00111111b
               db      00111111b

               db      00001100b   ; Cell (1,1)
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b

               db      00000011b   ; Cell (1,2)
               db      00000011b
               db      00000011b
               db      00000011b
               db      00000011b
               db      00000011b
               db      00000011b
               db      00000011b

               db      11001100b   ; Cell (2,0)
               db      11001100b
               db      11111100b
               db      11111100b
               db      11111111b
               db      11111111b
               db      00111111b
               db      00111111b

               db      11001100b   ; Cell (2,1)
               db      11001100b
               db      11111111b
               db      11111111b
               db      11000000b
               db      11000000b
               db      00000000b
               db      00000000b

               db      00000011b   ; Cell (2,2)
               db      00000011b
               db      11111111b
               db      11111111b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

; "M" (Width 3, Height 3)

MData:         db      00111111b   ; Cell (0,0)
               db      00111111b
               db      11111111b
               db      11111111b
               db      11111100b
               db      11111100b
               db      11001100b
               db      11001100b

               db      00000000b   ; Cell (0,1)
               db      00000000b
               db      11000000b
               db      11000000b
               db      11111111b
               db      11111111b
               db      11001100b
               db      11001100b

               db      00000000b   ; Cell (0,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      11111111b
               db      11111111b
               db      11000011b
               db      11000011b

               db      00111111b   ; Cell (1,0)
               db      00111111b
               db      00001100b
               db      00001100b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00001100b   ; Cell (1,1)
               db      00001100b
               db      00110000b
               db      00110000b
               db      00110000b
               db      00110000b
               db      11000000b
               db      11000000b

               db      11000011b   ; Cell (1,2)
               db      11000011b
               db      11000011b
               db      11000011b
               db      11000011b
               db      11000011b
               db      11000011b
               db      11000011b

               db      00000000b   ; Cell (2,0)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      11000000b   ; Cell (2,0)
               db      11000000b
               db      11000000b
               db      11000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      11000011b   ; Cell (2,2)
               db      11000011b
               db      11000011b
               db      11000011b
               db      00001111b
               db      00001111b
               db      00000000b
               db      00000000b

; "P" (Width 3, Height 5)

PData:         db      00111111b   ; Cell (0,0)
               db      00111111b
               db      11111100b
               db      11111100b
               db      11111111b
               db      11111111b
               db      11110000b
               db      11110000b

               db      00000000b   ; Cell (0,1)
               db      00000000b
               db      11000000b
               db      11000000b
               db      11110000b
               db      11110000b
               db      11000000b
               db      11000000b

               db      00000000b   ; Cell (0,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00111111b   ; Cell (1,0)
               db      00111111b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001111b
               db      00001111b

               db      00001100b   ; Cell (1,1)
               db      00001100b
               db      00110011b
               db      00110011b
               db      11000000b
               db      11000000b
               db      00000000b
               db      00000000b

               db      00000000b   ; Cell (1,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      11000000b
               db      11000000b
               db      00110000b
               db      00110000b

               db      00001111b   ; Cell (2,0)
               db      00001111b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b

               db      00000000b   ; Cell (2,1)
               db      00000000b
               db      11000000b
               db      11000000b
               db      00110011b
               db      00110011b
               db      00001100b
               db      00001100b

               db      00110000b   ; Cell (2,2)
               db      00110000b
               db      11000000b
               db      11000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00001100b   ; Cell (3,0)
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b

               db      00000000b   ; Cell (3,1)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00000000b   ; Cell (3,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00001100b   ; Cell (4,0)
               db      00001100b
               db      00001111b
               db      00001111b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00000000b   ; Cell (4,1)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00000000b   ; Cell (4,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

; "I" (Width 2, Height 4)

IData:         db      00001111b   ; Cell (0,0)
               db      00001111b
               db      00110011b
               db      00110011b
               db      00111100b
               db      00111100b
               db      00110000b
               db      00110000b

               db      11000000b   ; Cell (0,1)
               db      11000000b
               db      00110000b
               db      00110000b
               db      11110000b
               db      11110000b
               db      00110000b
               db      00110000b

               db      00001111b   ; Cell (1,0)
               db      00001111b
               db      00000011b
               db      00000011b
               db      00000011b
               db      00000011b
               db      00000011b
               db      00000011b

               db      11000000b   ; Cell (1,1)
               db      11000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00000011b   ; Cell (2,0)
               db      00000011b
               db      00000011b
               db      00000011b
               db      00000011b
               db      00000011b
               db      00000011b
               db      00000011b

               db      00000000b   ; Cell (2,1)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00000011b   ; Cell (3,0)
               db      00000011b
               db      00001111b
               db      00001111b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00000000b   ; Cell (3,1)
               db      00000000b
               db      11000000b
               db      11000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

; Cell (0,0)

NData:         db      00111111b   ; "N" (Width 3, Height 3)
               db      00111111b
               db      11111111b
               db      11111111b
               db      11111100b
               db      11111100b
               db      11001100b
               db      11001100b

               db      00000000b   ; Cell (0,1)
               db      00000000b
               db      11000000b
               db      11000000b
               db      11111111b
               db      11111111b
               db      11001100b
               db      11001100b

               db      00000000b   ; Cell (0,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      11111111b
               db      11111111b
               db      00000011b
               db      00000011b

               db      00111111b   ; Cell (1,0)
               db      00111111b
               db      00001100b
               db      00001100b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00001100b   ; Cell (1,1)
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b

               db      00000011b   ; Cell (1,2)
               db      00000011b
               db      00000011b
               db      00000011b
               db      00000011b
               db      00000011b
               db      00000011b
               db      00000011b

               db      00000000b   ; Cell (2,0)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00001100b   ; Cell (2,1)
               db      00001100b
               db      00001100b
               db      00001100b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00000011b   ; Cell (2,2)
               db      00000011b
               db      00000011b
               db      00000011b
               db      00001111b
               db      00001111b
               db      00000000b
               db      00000000b

; "G" (Width 3, Height 4)

GData:         db      00000000b   ; Cell (0,0)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00001111b   ; Cell (0,1)
               db      00001111b
               db      00110011b
               db      00110011b
               db      11111111b
               db      11111111b
               db      00110000b
               db      00110000b

               db      11000000b   ; Cell (0,2)
               db      11000000b
               db      11110000b
               db      11110000b
               db      11110000b
               db      11110000b
               db      11110000b
               db      11110000b

               db      00000011b   ; Cell (1,0)
               db      00000011b
               db      00001100b
               db      00001100b
               db      00110000b
               db      00110000b
               db      11000000b
               db      11000000b

               db      00001111b   ; Cell (1,1)
               db      00001111b
               db      11000011b
               db      11000011b
               db      00110011b
               db      00110011b
               db      00001111b
               db      00001111b

               db      11000000b   ; Cell (1,2)
               db      11000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      11000000b   ; Cell (2,0)
               db      11000000b
               db      00110000b
               db      00110000b
               db      00001100b
               db      00001100b
               db      00000011b
               db      00000011b

               db      00001111b   ; Cell (2,1)
               db      00001111b
               db      00110011b
               db      00110011b
               db      11000011b
               db      11000011b
               db      00000011b
               db      00000011b

               db      00000000b   ; Cell (2,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00000000b   ; Cell (3,0)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00110000b
               db      00110011b
               db      00111111b
               db      00111100b

               db      00000011b   ; Cell (3,1)
               db      00000011b
               db      00000011b
               db      00000011b
               db      00111111b
               db      11111111b
               db      11000000b
               db      00000000b

               db      00000000b   ; Cell (3,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

; "A" (Width 2, Height 4)

AData:         db      00000000b   ; Cell (0,0)
               db      00000000b
               db      00000011b
               db      00000011b
               db      00001111b
               db      00001111b
               db      00000011b
               db      00000011b

               db      11111100b   ; Cell (0,1)
               db      11111100b
               db      00111111b
               db      00111111b
               db      11111111b
               db      11111111b
               db      00001111b
               db      00001111b

               db      00000000b   ; Cell (1,0)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000011b
               db      00000011b
               db      00001100b
               db      00001100b

               db      11111100b   ; Cell (1,1)
               db      11111100b
               db      00110000b
               db      00110000b
               db      11110000b
               db      11110000b
               db      00110000b
               db      00110000b

               db      00110000b   ; Cell (2,0)
               db      00110000b
               db      00110000b
               db      00110000b
               db      11000000b
               db      11000000b
               db      11000000b
               db      11000000b

               db      00110000b   ; Cell (2,1)
               db      00110000b
               db      00110000b
               db      00110000b
               db      00110000b
               db      00110000b
               db      00110000b
               db      00110000b

               db      11000000b   ; Cell (3,0)
               db      11000000b
               db      11111111b
               db      11111111b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00110000b   ; Cell (3,1)
               db      00110000b
               db      11111100b
               db      11111100b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

; "C" (Width 3, Height 4)

CData:         db      00111111b   ; Cell (0,0)
               db      00111111b
               db      11111100b
               db      11111100b
               db      11111111b
               db      11111111b
               db      11110000b
               db      11110000b

               db      00000000b   ; Cell (0,1)
               db      00000000b
               db      11000000b
               db      11000000b
               db      11110000b
               db      11110000b
               db      11000000b
               db      11000000b

               db      00000000b   ; Cell (0,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00111111b   ; Cell (1,0)
               db      00111111b
               db      00001100b
               db      00001100b
               db      00001111b
               db      00001111b
               db      00001100b
               db      00001100b

               db      00000000b   ; Cell (1,1)
               db      00000000b
               db      00000000b
               db      00000000b
               db      11111111b
               db      11111111b
               db      00000000b
               db      00000000b

               db      00000000b   ; Cell (1,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00001100b   ; Cell (2,0)
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b

               db      00000000b   ; Cell (2,1)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00000000b   ; Cell (2,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00001100b   ; Cell (3,0)
               db      00001100b
               db      00001111b
               db      00001111b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00000000b   ; Cell (3,1)
               db      00000000b
               db      11111111b
               db      11111111b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      11000000b   ; Cell (3,2)
               db      11000000b
               db      11000000b
               db      11000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

; "K" (Width 3, Height 4)

KData:         db      00111111b   ; Cell (0,0)
               db      00111111b
               db      11111100b
               db      11111100b
               db      11111111b
               db      11111111b
               db      11110000b
               db      11110000b

               db      00000000b   ; Cell (0,1)
               db      00000000b
               db      11000000b
               db      11000000b
               db      11110000b
               db      11110000b
               db      11000000b
               db      11000000b

               db      00000000b   ; Cell (0,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00111111b   ; Cell (1,0)
               db      00111111b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b

               db      00000000b   ; Cell (1,1)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000011b
               db      00001111b
               db      00111100b
               db      11110000b

               db      00000000b   ; Cell (1,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00001111b   ; Cell (2,0)
               db      00001111b
               db      00001100b
               db      00001100b
               db      00001111b
               db      00001111b
               db      00001100b
               db      00001100b

               db      11000000b   ; Cell (2,1)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      11000000b
               db      11000000b

               db      00000000b   ; Cell (2,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b

               db      00001100b   ; Cell (3,0)
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001100b
               db      00001111b
               db      00001111b

               db      00110000b   ; Cell (3,1)
               db      00110000b
               db      00001100b
               db      00001100b
               db      00000011b
               db      00000011b
               db      00000000b
               db      00000000b

               db      00000000b   ; Cell (3,2)
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      00000000b
               db      11110000b
               db      11110000b




; Clears the screen and sets the desired attributes

InitScreen:    ld      a,07h
               out     (0feh),a    ; Set the border to white

               ld      a,00111001b ; A=Attributes (Paper White, Ink Blue)
               call    ClrScreen
               ret




ClrScreen:     ld      hl,4000h    ; HL points to screen memory
               ld      bc,17ffh    ; Set counter to size of screen memory
               ld      de,4001h    ; DE points to second byte of screen memory
               ld      (hl),00h    ; Zero first byte of screen memory
               ldir                ; Repeat (Zero screen memory)

               ; Set attributes

               ld      hl,5800h    ; HL points to display attributes
               ld      bc,02ffh    ; Set counter to size of attributes
               ld      de,5801h    ; DE points to second byte of attributes
               ld      (hl),a      ; Set first byte of attributes
               ldir                ; Repeat (Fill display attributes)
               ret




; PrintLet     Prints/erases a multi-cell sprite to/from the screen
;
; This routine is used to print & erase the big gfx letters. Pixels are XOR'd
; onto the screen - so reprinting to the same location will erase an image.
;
; This is also used to print the credits text (where each character is treated
; as a 1x1 cell sprite).
;
; ENTRY:
; HL = Pointer to GFX data to be printed
; DE = Pointer into Display memory - where to print top left cell
; BC = Size of GFX in cells (B=Width, C=Height)

PrintLet:      push    de          ; Save a copy of current print location
               push    bc          ; Save a copy of the width, height counter

NextHCell:     push    bc          ; Another copy (for width)

               ; Starting painting a cell

               ld      b,08h       ; 8 rows of pixels in this cell

NextRow:       ld      c,(hl)      ; C=pixel data for this row
               ld      a,(de)      ; A=pixels currently on screen at position
               xor     c           ; Turn on/off pixels as required
               ld      (de),a      ; Copy back to Display memory
               inc     hl          ; HL points to next row of pixel data
               inc     d           ; Move Display pointer to next pixel row
               djnz    NextRow     ; Print next pixel row

               ; We've painted a complete cell.

               pop     bc          ; Retrieve width counter (B)
               ld      a,d
               sub     08h
               ld      d,a         ; Return Display pointer (DE) to top of cell
               inc     de          ; Point DE to top of adjacent right cell
               djnz    NextHCell   ; Paint next horizontal cell

               ; We've finished painting a row of cells.

               pop     bc          ; BC= Width, Height counter
               pop     de          ; Reset DE to top of left-most cell in row
               ld      a,e
               add     a,20h
               ld      e,a         ; Point DE to top of cell immediately below
               jr      nc,Bnd_OK   ; Did we cross screen-third boundary?

Bnd_Cross:     ld      a,d         ; If so, adjust DE to point to intended cell
               add     a,08h       ; i.e add 2048 bytes (800h)
               ld      d,a

Bnd_OK:        dec     c           ; Decrement height counter
               jr      nz,PrintLet ; Paint next row of cells if req

               ; We've finished painting the last row of cells.

               ret




; SFXPause     Generates sound effect and briefly pauses
;
; This is called in between every frame of animation.  Depending on which frame
; has just been drawn, a different note is played.  A short delay is then
; generated before returning.

SFXPause:      ld      hl,BeepCnt  ; Variable to track current frame
               inc     (hl)        ;
               ld      a,(hl)      ; We're only interested in last 3 bits, i.e
               and     07h         ; counter will return to 0 after 7
               rlca
               rlca                ; A=A*4 (this will be the offset to jump)
               ld      hl,LitPause
               push    hl          ; Make this the return address after beep
               ld      (JpOff+1),a ; Alter the relative jump code to target
JpOff:         jr      JpOff       ; the appropriate routine for current frame

               jp      F1Beep
               nop
               jp      F2Beep
               nop
               jp      F3Beep
               nop
               jp      F4Beep
               nop
               jp      F5Beep
               nop
               jp      F6Beep
               nop
               jp      F7Beep
               nop
               jp      F8Beep

               ; Beep done.  Insert delay for animation before returning.

LitPause:      ld      hl,096bh
               call    Pause
               ret




; Pause        Generates a software delay
;
; This is used to insert brief pauses (of varying durations) into the animation.
;
; ENTRY
; HL = Number of times to loop

Pause:         dec     hl
               ld      a,l
               or      h
               jr      nz,Pause
               ret




; Various Beeps depending on which frame we're in

F1Beep:        ld      hl,0690h
               ld      de,0002h
               jp      Beep

F2Beep:        ld      hl,0584h
               ld      de,0002h
               jp      Beep

F3Beep:        ld      hl,04a3h
               ld      de,0002h
               jp      Beep

F4Beep:        ld      hl,03e6h
               ld      de,0003h
               jp      Beep

F5Beep:        ld      hl,0347h
               ld      de,0003h
               jp      Beep

F6Beep:        ld      hl,02c2h
               ld      de,0004h
               jp      Beep

F7Beep:        ld      hl,0252h
               ld      de,0005h
               jp      Beep

F8Beep:        ld      hl,01f4h
               ld      de,0006h
               jp      Beep




; Beep         Plays a note on the Spectrum's Beeper
;
; ENTRY
; HL = Pitch
; DE = Duration

Beep:          push    ix
               call    ROM_BEEP    ; ROM Beeper routine
               pop     ix
               ret




; DATA

BeepCnt:       db      0ffh        ; Used by SFXPause routine to keep track of
                                   ; current animation frame




; DATA - Animation
;
; There are 9 frames in each animation to bring the groups of big letters to
; their final positions.  The data below gives the addresses in screen memory
; of where to draw each letter for each frame.  The addresses point to the
; intended location for each letter's top left cell.

Frame0:        dw      506dh       ; "J" AT 19,13
               dw      506dh       ; "U" AT 19,13
               dw      50afh       ; "M" AT 21,15
               dw      5070h       ; "P" AT 19,16
               dw      508dh       ; "I" AT 20,13
               dw      50abh       ; "N" AT 21,11
               dw      508fh       ; "G" AT 20,15
               dw      506eh       ; "J" AT 19,14
               dw      508dh       ; "A" AT 20,13
               dw      508dh       ; "C" AT 20,13
               dw      508bh       ; "K" AT 20,11

Frame1:        dw      504ch       ; "J" AT 18,12
               dw      500dh       ; "U" AT 16,13
               dw      508fh       ; "M" AT 20,15
               dw      5070h       ; "P" AT 19,16
               dw      506bh       ; "I" AT 19,11
               dw      508ch       ; "N" AT 20,12
               dw      508eh       ; "G" AT 20,14
               dw      506dh       ; "J" AT 19,13
               dw      508ch       ; "A" AT 20,12
               dw      508fh       ; "C" AT 20,15
               dw      508dh       ; "K" AT 20,13

Frame2:        dw      48ebh       ; "J" AT 15,11
               dw      48ach       ; "U" AT 13,12
               dw      502eh       ; "M" AT 17,14
               dw      5050h       ; "P" AT 18,16
               dw      502bh       ; "I" AT 17,11
               dw      502ch       ; "N" AT 17,12
               dw      504eh       ; "G" AT 18,14
               dw      506ch       ; "J" AT 19,12
               dw      508dh       ; "A" AT 20,13
               dw      506fh       ; "C" AT 19,15
               dw      5071h       ; "K" AT 19,17

Frame3:        dw      488ah       ; "J" AT 12,10
               dw      486bh       ; "U" AT 11,11
               dw      48edh       ; "M" AT 15,13
               dw      5010h       ; "P" AT 16,16
               dw      48cch       ; "I" AT 14,12
               dw      48cdh       ; "N" AT 14,13
               dw      48efh       ; "G" AT 15,15
               dw      502dh       ; "J" AT 17,13
               dw      506eh       ; "A" AT 19,14
               dw      5030h       ; "C" AT 17,16
               dw      5032h       ; "K" AT 17,18

Frame4:        dw      4809h       ; "J" AT 8,9
               dw      482ah       ; "U" AT 9,10
               dw      488ch       ; "M" AT 12,12
               dw      488fh       ; "P" AT 12,15
               dw      486ch       ; "I" AT 11,12
               dw      488eh       ; "N" AT 12,14
               dw      48b0h       ; "G" AT 13,16
               dw      48eeh       ; "J" AT 15,14
               dw      502fh       ; "A" AT 17,15
               dw      48f1h       ; "C" AT 15,17
               dw      48f3h       ; "K" AT 15,19

Frame5:        dw      40a7h       ; "J" AT 5,7
               dw      4808h       ; "U" AT 8,8
               dw      480bh       ; "M" AT 8,11
               dw      482eh       ; "P" AT 9,14
               dw      480dh       ; "I" AT 8,13
               dw      482fh       ; "N" AT 9,15
               dw      4851h       ; "G" AT 10,17
               dw      48aeh       ; "J" AT 13,14
               dw      48f0h       ; "A" AT 15,16
               dw      48b2h       ; "C" AT 13,18
               dw      48b5h       ; "K" AT 13,21

Frame6:        dw      4065h       ; "J" AT 3,5
               dw      40c7h       ; "U" AT 6,7
               dw      40aah       ; "M" AT 5,10
               dw      40cdh       ; "P" AT 6,13
               dw      40aeh       ; "I" AT 5,14
               dw      40d0h       ; "N" AT 6,16
               dw      40f2h       ; "G" AT 7,18
               dw      486fh       ; "J" AT 11,15
               dw      48b1h       ; "A" AT 13,17
               dw      4874h       ; "C" AT 11,20
               dw      4877h       ; "K" AT 11,23

Frame7:        dw      4023h       ; "J" AT 1,3
               dw      4085h       ; "U" AT 4,5
               dw      4069h       ; "M" AT 3,9
               dw      408ch       ; "P" AT 4,12
               dw      406fh       ; "I" AT 3,15
               dw      4091h       ; "N" AT 4,17
               dw      4094h       ; "G" AT 4,20
               dw      4830h       ; "J" AT 9,16
               dw      4873h       ; "A" AT 11,19
               dw      4856h       ; "C" AT 10,22
               dw      4839h       ; "K" AT 9,25

Frame8:        dw      4000h       ; "J" AT 0,0
               dw      4063h       ; "U" AT 3,3
               dw      4047h       ; "M" AT 2,7
               dw      404bh       ; "P" AT 2,11
               dw      402fh       ; "I" AT 1,15
               dw      4052h       ; "N" AT 2,18
               dw      4076h       ; "G" AT 3,22
               dw      40f1h       ; "J" AT 7,17
               dw      4835h       ; "A" AT 9,21
               dw      4838h       ; "C" AT 9,24
               dw      481bh       ; "K" AT 8,27




; Entry Point (From BASIC Line 2)

Start:         call    InitScreen  ; Clear screen & set attributes

               ld      a,0ffh
               ld      (BeepCnt),a ; Reset counter used in SFXPause

               ld      ix,Frame0   ; Point IX to 1st frame positions and print
               call    PrintAll    ; all the letters

               ld      hl,7d00h
               call    Pause       ; Pause briefly - duration set by HL


; Animation begins here - groups of the big GFX letters are repeatedly drawn and
; erased through 8 frames until they reach their final positions.
; After each frame is drawn, SFXPause generates a sound effect and pauses before
; the next frame.

               ; Animate "M" & "A"

               call    PrintM      ; Erase Frame 0
               call    PrintA

               ld      ix,Frame1   ; Draw Frame 1
               call    PrintM
               call    PrintA
               call    SFXPause

               call    PrintA      ; Erase Frame 1
               call    PrintM

               ld      ix,Frame2   ; Draw Frame 2
               call    PrintM
               call    PrintA
               call    SFXPause

               call    PrintM      ; Erase Frame 2
               call    PrintA

               ld      ix,Frame3   ; Draw Frame 3
               call    PrintM
               call    PrintA
               call    SFXPause

               call    PrintA      ; Erase Frame 3
               call    PrintM

               ld      ix,Frame4   ; Draw Frame 4
               call    PrintM
               call    PrintA
               call    SFXPause

               call    PrintM      ; Erase Frame 4
               call    PrintA

               ld      ix,Frame5   ; Draw Frame 5
               call    PrintM
               call    PrintA
               call    SFXPause

               call    PrintA      ; Erase Frame 5
               call    PrintM

               ld      ix,Frame6   ; Draw Frame 6
               call    PrintM
               call    PrintA
               call    SFXPause

               call    PrintM      ; Erase Frame 6
               call    PrintA

               ld      ix,Frame7   ; Draw Frame 7
               call    PrintM
               call    PrintA
               call    SFXPause

               call    PrintM      ; Erase Frame 7
               call    PrintA

               ld      ix,Frame8   ; Draw Frame 8
               call    PrintM
               call    PrintA
               call    SFXPause

               ld      hl,7d00h
               call    Pause       ; Longer pause - duration set by HL

               ; Animate "I" & "K"

               ld      ix,Frame0   ; Erase Frame 0
               call    PrintI
               call    PrintK

               ld      ix,Frame1   ; Draw Frame 1
               call    PrintI
               call    PrintK
               call    SFXPause

               call    PrintI      ; Erase Frame 1
               call    PrintK

               ld      ix,Frame2   ; Draw Frame 2
               call    PrintI
               call    PrintK
               call    SFXPause

               call    PrintI      ; Erase Frame 2
               call    PrintK

               ld      ix,Frame3   ; Draw Frame 3
               call    PrintI
               call    PrintK
               call    SFXPause

               call    PrintI      ; Erase Frame 3
               call    PrintK

               ld      ix,Frame4   ; Draw Frame 4
               call    PrintI
               call    PrintK
               call    SFXPause

               call    PrintI      ; Erase Frame 4
               call    PrintK

               ld      ix,Frame5   ; Draw Frame 5
               call    PrintI
               call    PrintK
               call    SFXPause

               call    PrintI      ; Erase Frame 5
               call    PrintK

               ld      ix,Frame6   ; Draw Frame 6
               call    PrintI
               call    PrintK
               call    SFXPause

               call    PrintI      ; Erase Frame 6
               call    PrintK

               ld      ix,Frame7   ; Draw Frame 7
               call    PrintI
               call    PrintK
               call    SFXPause

               call    PrintI      ; Erase Frame 7
               call    PrintK

               ld      ix,Frame8   ; Draw Frame 8
               call    PrintI
               call    PrintK
               call    SFXPause

               ld      hl,7d00h
               call    Pause       ; Longer pause - duration set by HL

               ; Animate "J"(1), "P" & "C"

               ld      ix,Frame0   ; Erase Frame 0
               call    PrintJ1
               call    PrintP
               call    PrintC

               ld      ix,Frame1   ; Draw Frame 1
               call    PrintJ1
               call    PrintP
               call    PrintC
               call    SFXPause

               call    PrintJ1     ; Erase Frame 1
               call    PrintP
               call    PrintC

               ld      ix,Frame2   ; Draw Frame 2
               call    PrintJ1
               call    PrintP
               call    PrintC
               call    SFXPause

               call    PrintJ1     ; Erase Frame 2
               call    PrintP
               call    PrintC

               ld      ix,Frame3   ; Draw Frame 3
               call    PrintJ1
               call    PrintP
               call    PrintC
               call    SFXPause

               call    PrintJ1     ; Erase Frame 3
               call    PrintP
               call    PrintC

               ld      ix,Frame4   ; Draw Frame 4
               call    PrintJ1
               call    PrintP
               call    PrintC

               call    SFXPause    ; Erase Frame 4
               call    PrintJ1
               call    PrintP
               call    PrintC

               ld      ix,Frame5   ; Draw Frame 5
               call    PrintJ1
               call    PrintP
               call    PrintC
               call    SFXPause

               call    PrintJ1     ; Erase Frame 5
               call    PrintP
               call    PrintC

               ld      ix,Frame6   ; Draw Frame 6
               call    PrintJ1
               call    PrintP
               call    PrintC
               call    SFXPause

               call    PrintJ1     ; Erase Frame 6
               call    PrintP
               call    PrintC

               ld      ix,Frame7   ; Draw Frame 7
               call    PrintJ1
               call    PrintP
               call    PrintC
               call    SFXPause

               call    PrintJ1     ; Erase Frame 7
               call    PrintP
               call    PrintC

               ld      ix,Frame8   ; Draw Frame 8
               call    PrintJ1
               call    PrintP
               call    PrintC
               call    SFXPause

               ld      hl,7d00h
               call    Pause       ; Longer pause - duration set by HL

               ; Animate "U" & "G"

               ld      ix,Frame0   ; Erase Frame 0
               call    PrintU
               call    PrintG

               ld      ix,Frame1   ; Draw Frame 1
               call    PrintU
               call    PrintG
               call    SFXPause

               call    PrintU      ; Erase Frame 1
               call    PrintG

               ld      ix,Frame2   ; Draw Frame 2
               call    PrintU
               call    PrintG
               call    SFXPause

               call    PrintU      ; Erase Frame 2
               call    PrintG

               ld      ix,Frame3   ; Draw Frame 3
               call    PrintU
               call    PrintG
               call    SFXPause

               call    PrintU      ; Erase Frame 3
               call    PrintG

               ld      ix,Frame4   ; Draw Frame 4
               call    PrintU
               call    PrintG
               call    SFXPause

               call    PrintU      ; Erase Frame 4
               call    PrintG

               ld      ix,Frame5   ; Draw Frame 5
               call    PrintU
               call    PrintG
               call    SFXPause

               call    PrintU      ; Erase Frame 5
               call    PrintG

               ld      ix,Frame6   ; Draw Frame 6
               call    PrintU
               call    PrintG
               call    SFXPause

               call    PrintU      ; Erase Frame 6
               call    PrintG

               ld      ix,Frame7   ; Draw Frame 7
               call    PrintU
               call    PrintG
               call    SFXPause

               call    PrintU      ; Erase Frame 7
               call    PrintG

               ld      ix,Frame8   ; Draw Frame 8
               call    PrintU
               call    PrintG
               call    SFXPause

               ld      hl,7d00h
               call    Pause       ; Longer pause - duration set by HL

               ; Animate "N" & "J"(2)

               ld      ix,Frame0   ; Erase Frame 0
               call    PrintN
               call    PrintJ2

               ld      ix,Frame1   ; Draw Frame 1
               call    PrintN
               call    PrintJ2
               call    SFXPause

               call    PrintN      ; Erase Frame 1
               call    PrintJ2

               ld      ix,Frame2   ; Draw Frame 2
               call    PrintN
               call    PrintJ2
               call    SFXPause

               call    PrintN      ; Erase Frame 2
               call    PrintJ2

               ld      ix,Frame3   ; Draw Frame 3
               call    PrintN
               call    PrintJ2
               call    SFXPause

               call    PrintN      ; Erase Frame 3
               call    PrintJ2

               ld      ix,Frame4   ; Draw Frame 4
               call    PrintN
               call    PrintJ2
               call    SFXPause

               call    PrintN      ; Erase Frame 4
               call    PrintJ2

               ld      ix,Frame5   ; Draw Frame 5
               call    PrintN
               call    PrintJ2
               call    SFXPause

               call    PrintN      ; Erase Frame 5
               call    PrintJ2

               ld      ix,Frame6   ; Draw Frame 6
               call    PrintN
               call    PrintJ2
               call    SFXPause

               call    PrintN      ; Erase Frame 6
               call    PrintJ2

               ld      ix,Frame7   ; Draw Frame 7
               call    PrintN
               call    PrintJ2
               call    SFXPause

               call    PrintN      ; Erase Frame 7
               call    PrintJ2

               ld      ix,Frame8   ; Draw Frame 8
               call    PrintN
               call    PrintJ2
               call    SFXPause

               ld      hl,0fde8h
               call    Pause       ; Longer pause - duration set by HL


               ; Animation complete.  Now the credits and loading message are printed.

               ld      hl,Text1    ; "BY"
               ld      de,48cfh    ; Display location to print (AT 14,15)
               ld      b,02h       ; 2 characters to print
               call    PrintTxt

               ld      hl,Text2    ; "ALBERT BALL"
               ld      de,48eah    ; Display location to print (AT 15,10)
               ld      b,0bh       ; 11 characters to print
               call    PrintTxt

               ld      hl,Text3    ; "GRAPHICS BY STUART C BALL"
               ld      de,5024h    ; Display location to print (AT 17,4)
               ld      b,19h       ; 25 characters to print
               call    PrintTxt

               ld      hl,Text4    ; "COPYRIGHT: PROGRAM,AUDIO,VISUAL"
               ld      de,5060h    ; Display location to print (AT 19,0)
               ld      b,1fh       ; 31 characters to print
               call    PrintTxt

               ld      hl,Text5    ; "(c) 1983 BY IMAGINE SOFTWARE"
               ld      de,50c0h    ; Display location to print (AT 22,0)
               ld      b,1ah       ; 26 characters to print
               call    PrintTxt

               ; 33 bytes of NOPs here for unknown reason.

               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop


               ld      hl,Text6    ; "LOADING"
               ld      de,4860h    ; Display location to print (AT 11,0)
               ld      b,07h       ; 7 characters to print
               call    PrintTxt

               ; Set flash attribute for "LOADING" text

               ld      bc,07b9h    ; B=Number of cells to modify
                                   ; C=Attributes (Flash, Paper White, Ink Blue)
               ld      hl,5960h    ; HL=Points to cell (11,0) in Attribs memory

SetFlash:      ld      (hl),c      ; Set attrib byte
               inc     hl          ; Point to next cell
               djnz    SetFlash    ; Repeat for each letter


               ; Animation and text complete - now set up the system for the
               ; main game and load it.

               ld      sp,5c00h    ; Move stack to the printer buffer
               ld      hl,6f42h    ; 6f42h will be the start address for game so
               push    hl          ; make it the return address from Rom Loader
               ld      ix,5d8eh    ; Address to load bytes (23950)
               ld      de,2272h    ; Length of bytes to be loaded (8818)
               ld      a,0ffh      ; Data block (not header) to be loaded
               scf                 ; Load, not Verify.
               jp      ROM_LOAD    ; Call ROM LOADER




; The following 4 lines are never reached - here for unknown reason.

               call    PrintAll
               call    SFXPause
               call    PrintAll
               ret




PrintJ1:       ld      e,(ix+00h)
               ld      d,(ix+01h)  ; DE=Print location in display memory
               ld      hl,JData    ; HL=GFX data for big "J"
               ld      bc,0305h    ; BC=Width/Height of "J" in cells (3x5)
               call    PrintLet
               ret

PrintU:        ld      e,(ix+02h)
               ld      d,(ix+03h)  ; DE=Print location in display memory
               ld      hl,UData    ; HL=GFX data for big "U"
               ld      bc,0303h    ; BC=Width/Height of "U" in cells (3x3)
               call    PrintLet
               ret

PrintM:        ld      e,(ix+04h)
               ld      d,(ix+05h)  ; DE=Print location in display memory
               ld      hl,MData    ; HL=GFX data for big "M"
               ld      bc,0303h    ; BC=Width/Height of "M" in cells (3x3)
               call    PrintLet
               ret

PrintP:        ld      e,(ix+06h)
               ld      d,(ix+07h)  ; DE=Print location in display memory
               ld      hl,PData    ; HL=GFX data for big "P"
               ld      bc,0305h    ; BC=Width/Height of "P" in cells (3x5)
               call    PrintLet
               ret

PrintI:        ld      e,(ix+08h)
               ld      d,(ix+09h)  ; DE=Print location in display memory
               ld      hl,IData    ; HL=GFX data for big "I"
               ld      bc,0204h    ; BC=Width/Height of "I" in cells (2x4)
               call    PrintLet
               ret

PrintN:        ld      e,(ix+0ah)
               ld      d,(ix+0bh)  ; DE=Print location in display memory
               ld      hl,NData    ; HL=GFX data for big "N"
               ld      bc,0303h    ; BC=Width/Height of "N" in cells (3x3)
               call    PrintLet
               ret

PrintG:        ld      e,(ix+0ch)
               ld      d,(ix+0dh)  ; DE=Print location in display memory
               ld      hl,GData    ; HL=GFX data for big "G"
               ld      bc,0304h    ; BC=Width/Height of "G" in cells (3x4)
               call    PrintLet
               ret

PrintJ2:       ld      e,(ix+0eh)
               ld      d,(ix+0fh)  ; DE=Print location in display memory
               ld      hl,JData    ; HL=GFX data for big "J"
               ld      bc,0305h    ; BC=Width/Height of "J" in cells (3x5)
               call    PrintLet
               ret

PrintA:        ld      e,(ix+10h)
               ld      d,(ix+11h)  ; DE=Print location in display memory
               ld      hl,AData    ; HL=GFX data for big "A"
               ld      bc,0204h    ; BC=Width/Height of "A" in cells (2x4)
               call    PrintLet
               ret

PrintC:        ld      e,(ix+12h)
               ld      d,(ix+13h)  ; DE=Print location in display memory
               ld      hl,CData    ; HL=GFX data for big "C"
               ld      bc,0304h    ; BC=Width/Height of "C" in cells (3x4)
               call    PrintLet
               ret

PrintK:        ld      e,(ix+14h)
               ld      d,(ix+15h)  ; DE=Print location in display memory
               ld      hl,KData    ; HL=GFX data for big "K"
               ld      bc,0304h    ; BC=Width/Height of "K" in cells (3x4)
               call    PrintLet
               ret




PrintAll:      ; Routine to print all the letters

               call    PrintJ1
               call    PrintU
               call    PrintM
               call    PrintP
               call    PrintI
               call    PrintN
               call    PrintG
               call    PrintJ2
               call    PrintA
               call    PrintC
               call    PrintK
               ret




; PrintTxt     Prints a text string
;
; This routine prints a string of text by making calls to PrintLet for each
; character to be printed.
;
; ENTRY:
; HL = Pointer to string to be printed
; DE = Pointer to cell in Display memory - where to print first char
; B = Length of string

PrintTxt:      push    hl          ; Save copy of pointer to string
               push    bc          ; Save copy of string length
               push    de          ; Save copy of print location

               ; Character set pixel definitions are stored in ROM at (3c00h +
               ; 8*char code)

               ld      a,(hl)      ; A=Code of next character to be printed
               ld      hl,3c00h    ; HL points to base address
               ld      e,08h       ; Multiplication factor
               ld      d,00h

MultLoop:      add     hl,de       ; Add 8 to HL
               dec     a
               jr      nz,MultLoop ; Repeat 'A' times

               ; HL now points to ROM 8-byte pixel definition of char to be
               ; printed.

               ld      bc,0101h    ; It's a 1x1 cell GFX to be printed
               pop     de          ; DE points to print location in display
               push    de          ; Save a copy
               call    PrintLet

               pop     de          ; DE=Print location of cell just drawn
               inc     de          ; Point to next cell along.
               pop     bc          ; Length counter
               pop     hl          ; HL points to character just printed
               inc     hl          ; Point to next character to be printed
               djnz    PrintTxt    ; Print next character if req
               ret




Text1:         db      "BY"
Text2:         db      "ALBERT BALL"
Text3:         db      "GRAPHICS BY STUART C BALL"
Text4:         db      "COPYRIGHT: PROGRAM,AUDIO,VISUAL"
Text5:         db      7fh         ; Copyright symbol
               db      " 1983 BY IMAGINE SOFTWARE"
Text6:         db      "LOADING"




; 57 bytes of NOPs here for unknown reason.

               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
               nop
;===============================================================================